@@ -555,6 +555,38 @@ class TDWUtils:
         z = sy * cp * cr - cy * sp * sr
         return [x, y, z, w]
 
+    @staticmethod
+    def quaternion_to_euler_angles(quaternion: np.array) -> np.array:
+        """
+        :param quaternion: A quaternion as a nump array.
+
+        :return: The Euler angles representation of the quaternion.
+        """
+
+        x = quaternion[0]
+        y = quaternion[1]
+        z = quaternion[2]
+        w = quaternion[3]
+        ysqr = y * y
+
+        t0 = +2.0 * (w * x + y * z)
+        t1 = +1.0 - 2.0 * (x * x + ysqr)
+        ex = np.degrees(np.arctan2(t0, t1))
+
+        t2 = +2.0 * (w * y - z * x)
+        t2 = np.where(t2 > +1.0, +1.0, t2)
+        # t2 = +1.0 if t2 > +1.0 else t2
+
+        t2 = np.where(t2 < -1.0, -1.0, t2)
+        # t2 = -1.0 if t2 < -1.0 else t2
+        ey = np.degrees(np.arcsin(t2))
+
+        t3 = +2.0 * (w * z + x * y)
+        t4 = +1.0 - 2.0 * (ysqr + z * z)
+        ez = np.degrees(np.arctan2(t3, t4))
+
+        return np.array([ex, ey, ez])
+
     @staticmethod
     def get_unit_scale(record: ModelRecord) -> float:
         """
@@ -586,24 +618,19 @@ class TDWUtils:
         if new_config_path:
             config_path.write_text("[default]\nregion = us-east-1\noutput = json")
             print(f"Generated a new config file: {config_path.resolve()}")
-
         try:
             session = boto3.Session(profile_name="tdw")
             s3 = session.resource("s3")
-            tdw_private = False
-            for bucket in s3.buckets.all():
-                if bucket.name == "tdw-private":
-                    tdw_private = True
-            if not tdw_private:
-                print("ERROR! Could not access bucket tdw-private. Make sure you have the right permissions.")
-                return False
+            s3.meta.client.head_object(Bucket='tdw-private', Key='models/windows/2018-2019.1/iron_box')
             return True
         except ProfileNotFound:
             print(f"ERROR! Your AWS credentials file is not set up correctly.")
             print("Your AWS credentials must have a [tdw] profile with valid keys.")
             return False
-        except ClientError:
-            print("Error! Bad S3 credentials.")
+        except ClientError as e:
+            print("ERROR! Could not access bucket tdw-private. Make sure you have the right permissions.")
+            error_code = e.response['Error']['Code']
+            print(e, error_code)
             return False
 
     @staticmethod
